--
-- addMultiFruit
-- adds limited Multifruit capability to all vehicles
--
-- author: upsidedown
-- initial version: 03.09.2013
-- V0.9 PUBLIC BETA: 20.09.2013
-- V0.91/RC1: 20.10.2013 (several fixes, new fruits added)
-- V1.0: 21.10.2013 (added cutter/sowing differenciation)


addMultiFruit = {};
local addMultiFruit_directory = g_currentModDirectory;

function addMultiFruit.prerequisitesPresent(specializations)
	return true;
end;

-- function addMultiFruit:load(xmlFile)
-- end;

--function addMultiFruit:postLoad(xmlFile)
function addMultiFruit:load(xmlFile)
	self.removeDoublePS = SpecializationUtil.callSpecializationsFunction("removeDoublePS");
	
	local newFillTypes =        "silage forage sunflower soybean rice oat rye hemp millet dinkel triticale mohn greenwheat millet_windrow triticale_windrow millet_windrow dinkel_windrow oat_windrow rye_windrow hemp_windrow";
	local substituteFillTypes = "chaff silage rape maize maize barley barley barley barley barley barley rape barley barley_windrow barley_windrow barley_windrow barley_windrow barley_windrow barley_windrow barley_windrow barley_windrow"
	
	if self.preferedCombineSize ~= nil then --its a cutter
		substituteFillTypes = "chaff silage maize barley barley barley barley barley barley barley barley rape barley barley_windrow barley_windrow barley_windrow barley_windrow barley_windrow barley_windrow barley_windrow barley_windrow"
	end;
	
	if self.isSowingMachineFilling ~= nil then --its a sowingmachine
		substituteFillTypes = "chaff silage maize maize barley barley barley barley barley barley barley rape barley barley_windrow barley_windrow barley_windrow barley_windrow barley_windrow barley_windrow barley_windrow barley_windrow"
	end;
	
	
	self.addMultiFruitNewFillTypes = newFillTypes;
	self.addMultiFruitSubFillTypes = substituteFillTypes;
	
	if self.bodyTypes ~= nil then
		print("Multifruit:: has changeBodyTypes specialisation, no multi fruits added!")
		return;
	end;
	
	if self.varBody~= nil then 
		if self.varBody.nodes ~= nil then
		
			for k,entry in pairs(self.varBody.nodes) do
				local types = Utils.splitString(" ",newFillTypes);
				local subTypes = Utils.splitString(" ",substituteFillTypes);
				for k,v in pairs(types) do
					local fillType = Fillable.fillTypeNameToInt[v];
					local subType_new = Fillable.fillTypeNameToInt[subTypes[k]];
					if fillType ~= nil and subType_new ~= nil then --filltype and subtype are installed in map 
						if not entry.fillTypes[fillType] and entry.fillTypes[subType_new] then
							entry.fillTypes[fillType] = true; --add fillType 
							--print(v,"  ",subTypes[k])
							if entry.fillPlanes ~= nil then --add fillPlanes
								local subFill = subTypes[k];
								-- print(entry.fillPlanes[subFill] ~= nil)
								-- print(entry.fillPlanes[subFill] == nil)
								-- print(entry.fillPlanes[Fillable.fillTypeIntToName[fillType]] == nil)
								-- print(entry.fillPlanes[Fillable.fillTypeIntToName[fillType]] ~= nil)
								if entry.fillPlanes[subFill] ~= nil and entry.fillPlanes[Fillable.fillTypeIntToName[fillType]] == nil then
									--print("change fillplane")
									entry.fillPlanes[Fillable.fillTypeIntToName[fillType]] = entry.fillPlanes[subFill];
								else
									--print("no change of fillplane")
								end;
							end;
						end;
					end;
				end;
			end;
		end;
	end;
	
	
	if self.fillTypes ~= nil then
		local types = Utils.splitString(" ",newFillTypes);
		local subTypes = Utils.splitString(" ",substituteFillTypes);
		for k,v in pairs(types) do

			local fillType = Fillable.fillTypeNameToInt[v];
			local subType_new = Fillable.fillTypeNameToInt[subTypes[k]];
			if fillType ~= nil and subType_new ~= nil then --filltype and subtype are installed in map 
				
				
				if not self.fillTypes[fillType] and self.fillTypes[subType_new] then
					self.fillTypes[fillType] = true; --add fillType 
					--print(v,"  ",subTypes[k])
					if self.fillPlanes ~= nil then --add fillPlanes
						local subFill = subTypes[k];
						if self.fillPlanes[subFill] ~= nil and self.fillPlanes[Fillable.fillTypeIntToName[fillType]] == nil then
							self.fillPlanes[Fillable.fillTypeIntToName[fillType]] = self.fillPlanes[subFill];
						end;					
					end;
					
					
					if self.tipAnimations ~= nil then --add tipAnimation Particle Systems						
						for kkk,tipAnimation in pairs(self.tipAnimations) do
							local sub = Fillable.fillTypeNameToInt[subTypes[k]]
						
							if tipAnimation.dischargeParticleSystems[sub] ~= nil and tipAnimation.dischargeParticleSystems[fillType]==nil then
								tipAnimation.dischargeParticleSystems[fillType] = tipAnimation.dischargeParticleSystems[sub];
								tipAnimation.dischargeParticleSystems[fillType].original = sub;						
							end;
						end;
					end;
					
					
					
					if self.baleTypes ~= nil then --add 
						
						local subFill = Fillable.fillTypeNameToInt[subTypes[k]];
						if self.baleTypes[subFill] ~= nil and self.baleTypes[fillType] == nil then
							self.baleTypes[fillType] = self.baleTypes[subFill];
						end;					
						-- for fillType, enabled in pairs(self.fillTypes) do --repeat the post-load of baler.lua
							-- if enabled and fillType ~= Fillable.FILLTYPE_UNKNOWN then
								-- if FruitUtil.fillTypeIsWindrow[fillType] then
									-- table.insert(self.balerPickupFillTypes, fillType);
								-- end
							-- end
						-- end					

					end;
					
					
					
					
					if self.emptyParticleSystems ~= nil then --add 
						local subFill = Fillable.fillTypeNameToInt[subTypes[k]];
						if self.emptyParticleSystems[subFill] ~= nil and self.emptyParticleSystems[fillType] == nil then
							self.emptyParticleSystems[fillType] = self.emptyParticleSystems[subFill];
							self.emptyParticleSystems[fillType].original = subFill;
						end;					
					end;
					
					if self.shovelFillParticleSystems ~= nil then --add 
						local subFill = Fillable.fillTypeNameToInt[subTypes[k]];
						if self.shovelFillParticleSystems[subFill] ~= nil and self.shovelFillParticleSystems[fillType] == nil then
							self.shovelFillParticleSystems[fillType] = self.shovelFillParticleSystems[subFill];
							self.shovelFillParticleSystems[fillType].original = subFill;
						end;					
					end;
					

				end;
				
				
			else --fruit is not installed in map
			   --print("Warning: '"..self.configFileName.. "' has invalid fillType (MF mod)'"..v.."'.");
			end;
			
			
		end;
	end;
	
	--fruittypes for combines, cutters etc
	if true then
		local types = Utils.splitString(" ",newFillTypes);
		local subTypes = Utils.splitString(" ",substituteFillTypes);
		for k,v in pairs(types) do
			local descNew = FruitUtil.fruitTypes[v];
			local descSub = FruitUtil.fruitTypes[subTypes[k]];
			if descNew ~= nil and descSub ~= nil then
				
				if self.pipeParticleSystems~= nil then
					if self.pipeParticleSystems[descSub.index] ~= nil and self.pipeParticleSystems[descNew.index] == nil then
						self.pipeParticleSystems[descNew.index] = self.pipeParticleSystems[descSub.index];
						self.pipeParticleSystems[descNew.index].original = descSub.index;
					end;
				end;
				
				if self.convertedFruits~= nil then
					if self.convertedFruits[descSub.index] ~= nil and self.convertedFruits[descNew.index] == nil then
						self.convertedFruits[descNew.index] = self.convertedFruits[descSub.index]
					end;
				end;
				
				if self.grainTankPlanes~= nil then
					if self.grainTankPlanes[subTypes[k]] ~= nil and self.grainTankPlanes[v] == nil then
						self.grainTankPlanes[v] = self.grainTankPlanes[subTypes[k]];
					end;
				end;
				
				if self.chopperParticleSystems~= nil then
					if self.chopperParticleSystems[descSub.index] ~= nil and self.chopperParticleSystems[descNew.index] == nil then
						self.chopperParticleSystems[descNew.index] = self.chopperParticleSystems[descSub.index];
						self.chopperParticleSystems[descNew.index].original = descSub.index;
					end;
				end;
				
				if self.strawParticleSystems~= nil then
					if self.strawParticleSystems[descSub.index] ~= nil and self.strawParticleSystems[descNew.index] == nil then
						self.strawParticleSystems[descNew.index] = self.strawParticleSystems[descSub.index];
						self.strawParticleSystems[descNew.index].original = descSub.index;
					end;
				end;
				
				
				if self.combineThreshingFillParticleSystems~= nil then
					if self.combineThreshingFillParticleSystems[descSub.index] ~= nil and self.combineThreshingFillParticleSystems[descNew.index] == nil then
						self.combineThreshingFillParticleSystems[descNew.index] = self.combineThreshingFillParticleSystems[descSub.index];
						self.combineThreshingFillParticleSystems[descNew.index].original = descSub.index;
					end;
				end;
				
				if self.grainTankFruitTypes~= nil then
					if self.grainTankFruitTypes[descSub.index] ~= nil and self.grainTankFruitTypes[descNew.index] == nil then
						self.grainTankFruitTypes[descNew.index] = self.grainTankFruitTypes[descSub.index];
					end;
				end;
				
				
				if self.fruitExtraObjects~= nil then
					if self.fruitExtraObjects[subTypes[k]] ~= nil and self.fruitExtraObjects[v] == nil then
						self.fruitExtraObjects[v] = self.fruitExtraObjects[subTypes[k]];
					end;
				end;
				
				
				if self.fruitTypes~= nil then
					if self.fruitTypes[descSub.index] ~= nil then
						if self.fruitTypes[descNew.index] == nil then
							self.fruitTypes[descNew.index] = self.fruitTypes[descSub.index];
							-- print(descNew.index,"  ",descSub.index)
							-- print(descNew.name,"  ",descSub.name)
							-- print(self.fruitTypes[descNew.index])
							-- print("==============")
						end;
					end;
				end;
				
				
				--sfm combine:
				if self.fruitParticle ~= nil then
					if self.fruitParticle.leftFreePart ~= nil then
						if self.fruitParticle.leftFreePart[descSub.name] ~= nil and self.fruitParticle.leftFreePart[descNew.name] == nil then
							self.fruitParticle.leftFreePart[descNew.name] = self.fruitParticle.leftFreePart[descSub.name];
						end;
					end;				
				end
				
				if self.fruitParticle ~= nil then
					if self.fruitParticle.rightFreePart ~= nil then
						if self.fruitParticle.rightFreePart[descSub.name] ~= nil and self.fruitParticle.rightFreePart[descNew.name] == nil then
							self.fruitParticle.rightFreePart[descNew.name] = self.fruitParticle.rightFreePart[descSub.name];
						end;
					end;				
				end
				
				
				if self.seeds ~= nil then
					if descSub.allowsSeeding then
						for _,sub in pairs(self.seeds) do
							if descSub.index == sub then
								table.insert(self.seeds, descNew.index);
							end;
						end;
						
					end;
				end;
				
			end;
		end;
	
	end
	
	
	
end;

function addMultiFruit:delete()
end;

function addMultiFruit:mouseEvent(posX, posY, isDown, isUp, button)
end;

function addMultiFruit:keyEvent(unicode, sym, modifier, isDown)
end;

function addMultiFruit:update(dt)
	
	
end;


function addMultiFruit:updateTick(dt)
	-- if false then
		-- if self.fruitTypes ~= nil then
			-- for fruitId,_ in pairs(self.fruitTypes) do
				-- local desc = FruitUtil.fruitIndexToDesc[fruitId];
				-- if desc ~= nil then
					-- print("fruit: ",fruitId," cutstate: ",desc.cutState)
				-- else
					-- print(fruitId," desc == nil!")
				-- end;			
			-- end
		-- end	
	-- end
end;

function addMultiFruit:readStream(streamId, connection)

end;

function addMultiFruit:writeStream(streamId, connection)


end;

function addMultiFruit:draw()
	

end;




function addMultiFruit:preDelete()

	
	if self.tipAnimations ~= nil then	
		for k, tipAnimations in pairs(self.tipAnimations) do
			self:removeDoublePS(tipAnimations.dischargeParticleSystems)
		end;		
	end;

	
	self:removeDoublePS(self.emptyParticleSystems)
	self:removeDoublePS(self.shovelFillParticleSystems)
	self:removeDoublePS(self.pipeParticleSystems)
	self:removeDoublePS(self.chopperParticleSystems)
	self:removeDoublePS(self.strawParticleSystems)
	self:removeDoublePS(self.combineThreshingFillParticleSystems)	
end;


function addMultiFruit:removeDoublePS(PSlist)
	if PSlist ~= nil then
		for kk, particleSystem in pairs(PSlist) do
			if particleSystem.original~= nil then
				if particleSystem.original ~= kk then
					PSlist[kk] = nil
				end;
			end;
		end;
	end;
end;